<?php
/**
 * Authentification sur abuledu ... via pam
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Andreas Gohr <andi@splitbrain.org>
 * @author     Chris Smith <chris@jalakai.co.uk>
 * @author     Eric Seigne <eric.seigne@ryxeo.com>
 */
define('DOKU_AUTH', dirname(__FILE__));
require_once(DOKU_AUTH.'/basic.class.php');

define('AUTH_USERFILE',DOKU_CONF.'users.auth.php');

class auth_abuledu extends auth_basic {
  var $users = null;
  var $_pattern = array();

  /**
   * Constructor
   *
   * Carry out sanity checks to ensure the object is
   * able to operate. Set capabilities.
   *
   * @author  Eric Seigne <eric.seigne@ryxeo.com>
   */
  function auth_plain() {
    $this->cando['addUser']      = false;
    $this->cando['delUser']      = false;
    $this->cando['modLogin']     = false;
    $this->cando['modPass']      = false;
    $this->cando['modName']      = false;
    $this->cando['modMail']      = false;
    $this->cando['modGroups']    = false;
    $this->cando['getUsers']     = true;
    $this->cando['getUserCount'] = true;
  }
  
  /**
   * Check user+password [required auth function]
   *
   * Checks if the given user exists and the given
   * plaintext password is correct
   *
   * @author  Eric Seigne <eric.seigne@ryxeo.com>
   * @return  bool
   */
  function checkPass($user,$pass){
    $userinfo = $this->getUserData($user);
    if ($userinfo === false) return false;
    
    $desc = array(
		  0 => array("pipe","r"),
		  1 => array("pipe","w"),
		  2 => array("file","/tmp/error.out","a")
		  );
    
    $proc = proc_open('sudo /usr/bin/pam_auth -o -1',$desc, $pipes);
    
    if (is_resource($proc)) {
      fwrite($pipes[0],$user);
      fwrite($pipes[0]," ");
      fwrite($pipes[0],$pass);
      fwrite($pipes[0],"\n");
      fclose($pipes[0]);
      $retour = trim(stream_get_contents($pipes[1]));
      fclose($pipes[1]);
    }
    proc_close($proc);
    if($retour == "OK")
      return true;
    else
      return false;
  }
  
  /**
   * Return user info
   *
   * Returns info about the given user needs to contain
   * at least these fields:
   *
   * name string  full name of the user
   * mail string  email addres of the user
   * grps array   list of groups the user is in
   *
   * @author  Eric Seigne <eric.seigne@ryxeo.com>
   */
  function getUserData($user){
    if($this->users === null) $this->_loadUserData();
    return isset($this->users[$user]) ? $this->users[$user] : false;
  }
  
  /**
   * Create a new User
   *
   * Returns false if the user already exists, null when an error
   * occurred and true if everything went well.
   *
   * The new user will be added to the default group by this
   * function if grps are not specified (default behaviour).
   *
   * @author  Andreas Gohr <andi@splitbrain.org>
   * @author  Chris Smith <chris@jalakai.co.uk>
   */
  function createUser($user,$pwd,$name,$mail,$grps=null){
    return null;
  }

  /**
   * Modify user data
   *
   * @author  Chris Smith <chris@jalakai.co.uk>
   * @param   $user      nick of the user to be changed
   * @param   $changes   array of field/value pairs to be changed (password will be clear text)
   * @return  bool
   */
  function modifyUser($user, $changes) {
    return false;
  }

  /**
   *  Remove one or more users from the list of registered users
   *
   *  @author  Christopher Smith <chris@jalakai.co.uk>
   *  @param   array  $users   array of users to be deleted
   *  @return  int             the number of users deleted
   */
  function deleteUsers($users) {
    return 0;
  }

  /**
   * Return a count of the number of user which meet $filter criteria
   *
   * @author  Chris Smith <chris@jalakai.co.uk>
   */
  function getUserCount($filter=array()) {
    return 0;
  }

  /**
   * Bulk retrieval of user data
   *
   * @author  Chris Smith <chris@jalakai.co.uk>
   * @param   start     index of first user to be returned
   * @param   limit     max number of users to be returned
   * @param   filter    array of field/pattern pairs
   * @return  array of userinfo (refer getUserData for internal userinfo details)
   */
  function retrieveUsers($start=0,$limit=0,$filter=array()) {
    if ($this->users === null) $this->_loadUserData();
    ksort($this->users);
    
    $i = 0;
    $count = 0;
    $out = array();
    $this->_constructPattern($filter);
    
    foreach ($this->users as $user => $info) {
      if ($this->_filter($user, $info)) {
	if ($i >= $start) {
	  $out[$user] = $info;
	  $count++;
	  if (($limit > 0) && ($count >= $limit)) break;
	}
	$i++;
      }
    }
    
    return $out;
  }
  
  /**
   * Load all user data
   *
   * loads the user file into a datastructure
   *
   * @author  Andreas Gohr <andi@splitbrain.org>
   */
  function _loadUserData(){
    require_once("AbE_API.php");
    require_once("AbE_Users.php");
    $this->users = array();
    
    //tester pam-auth    if(!@file_exists(AUTH_USERFILE)) return;
    $lines = @file("/etc/passwd");
    foreach($lines as $line){
      $line = trim($line);
      if(empty($line)) continue;
      $row = explode(":",$line);
      $u = new AbE_Users();
      $u->GetUser($row[0]);
      $groupes = $u->GetGroupes(1);
      //print "<p>GRP: $line // " . $row[0] . " || $groupes</p>";
      $tabuser = posix_getpwnam($row[0]);

      $this->users[$row[0]]['pass'] = "";
      $this->users[$row[0]]['name'] = $tabuser["gecos"];
      $this->users[$row[0]]['mail'] = $tabuser["name"] . "@localhost";
      $this->users[$row[0]]['grps'] = $groupes;

      unset($u);
      unset($tabuser);
      unset($tabgrp);
    }
  }
  
  /**
   * return 1 if $user + $info match $filter criteria, 0 otherwise
   *
   * @author   Chris Smith <chris@jalakai.co.uk>
   */
  function _filter($user, $info) {
    return 0;
  }
  
  function _constructPattern($filter) {
    $this->_pattern = array();
    foreach ($filter as $item => $pattern) {
      //        $this->_pattern[$item] = '/'.preg_quote($pattern,"/").'/i';          // don't allow regex characters
      $this->_pattern[$item] = '/'.str_replace('/','\/',$pattern).'/i';    // allow regex characters
    }
  }
}

